package com.diaoyun.zion.chinafrica.service.impl;

import com.diaoyun.zion.chinafrica.constant.KeyConstant;
import com.diaoyun.zion.chinafrica.dao.TbCfCouponDao;
import com.diaoyun.zion.chinafrica.dao.TbCfTakeCouponDao;
import com.diaoyun.zion.chinafrica.entity.TbCfCouponEntity;
import com.diaoyun.zion.chinafrica.entity.TbCfTakeCouponEntity;
import com.diaoyun.zion.chinafrica.service.TbCfCouponService;
import com.diaoyun.zion.chinafrica.vo.TbCfUserInfoVo;
import com.diaoyun.zion.master.base.Result;
import com.diaoyun.zion.master.base.ResultCode;
import com.diaoyun.zion.master.common.RedisCache;
import com.diaoyun.zion.master.common.TokenManager;
import com.diaoyun.zion.master.util.CookieUtils;
import com.diaoyun.zion.master.util.IdUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 优惠券表Service实现类
 *
 * @author G
 * @date 2019-08-14 09:11:48
 */
@Service("tbCfCouponService")
public class TbCfCouponServiceImpl implements TbCfCouponService {
    @Autowired
    private TbCfCouponDao tbCfCouponDao;
    @Autowired
    private TbCfTakeCouponDao tbCfTakeCouponDao;

    @Resource(name = "redisTokenManager")
    private TokenManager tokenManager;

    @Autowired
    private HttpServletRequest request; //自动注入request
    @Resource
    private RedisCache<Map<String,TbCfCouponEntity>> redisCache;

    @Override
    public TbCfCouponEntity queryObject(String couponId) {
        return tbCfCouponDao.queryObject(couponId);
    }

    @Override
    public List<TbCfCouponEntity> queryList(Map<String, Object> map) {
        return tbCfCouponDao.queryList(map);
    }

    @Override
    public int queryTotal(Map<String, Object> map) {
        return tbCfCouponDao.queryTotal(map);
    }

    @Override
    public int save(TbCfCouponEntity tbCfCoupon) {
        tbCfCoupon.setCouponId(IdUtil.createIdbyUUID());
        return tbCfCouponDao.save(tbCfCoupon);
    }

    @Override
    public int update(TbCfCouponEntity tbCfCoupon) {
        return tbCfCouponDao.update(tbCfCoupon);
    }

    @Override
    public int delete(String couponId) {
        return tbCfCouponDao.delete(couponId);
    }

    @Override
    public int deleteBatch(String[] couponIds) {
        return tbCfCouponDao.deleteBatch(couponIds);
    }

    @Override
    public Result takeCoupon(String couponId) {
        Result result=new Result();
        //获取用户
        String token = CookieUtils.getCookie(request, TokenManager.TOKEN);
        TbCfUserInfoVo tbCfUserInfoVo = tokenManager.validate(token);
        //判断用户是否已经领取
        boolean takeFlag=repeatTakeCoupon(tbCfUserInfoVo.getUserId(),couponId);
        if(takeFlag) {
            result.setCode(ResultCode.ERROR).setMessage("你已经领取了此优惠券");
        } else {
            //redisCache.delete(KeyConstant.COUPON);
            //先从redis获取，若获取不到，则加入 TODO 修改的时候要同步修改redis
            Map<String,TbCfCouponEntity> couponMap=redisCache.get(KeyConstant.COUPON);
            TbCfCouponEntity tbCfCouponEntity ;
            if(couponMap==null) {
                tbCfCouponEntity =tbCfCouponDao.queryObject(couponId);
                couponMap=new HashMap<>();
                couponMap.put(couponId,tbCfCouponEntity);
                redisCache.set(KeyConstant.COUPON,couponMap);
            } else if(couponMap.get(couponId)==null) {
                tbCfCouponEntity =tbCfCouponDao.queryObject(couponId);
                couponMap.put(couponId,tbCfCouponEntity);
                redisCache.set(KeyConstant.COUPON,couponMap);
            } else {

                tbCfCouponEntity=couponMap.get(couponId);
            }
            //发放数量
            Integer quato=tbCfCouponEntity.getQuato();
            //已经领取数量
            Integer takeCount=tbCfCouponEntity.getTakeCount();
            //可领取
            if(quato>takeCount) {
                //先改动redis，再入库，一定程度上减少数据的覆盖
                tbCfCouponEntity.setTakeCount(tbCfCouponEntity.getTakeCount()+1);
                tbCfCouponEntity.setQuato(tbCfCouponEntity.getQuato()-1);
                couponMap.put(couponId,tbCfCouponEntity);
                redisCache.set(KeyConstant.COUPON,couponMap);
                //发放优惠券
                giveOutCoupon(tbCfUserInfoVo.getUserId(),tbCfCouponEntity);
                result.setMessage("领取成功");
            } else {
                result.setCode(ResultCode.ERROR).setMessage("已被抢光");
            }
        }

        return result;
    }

    /**
     * 判断用户是否已经领取
     * @param couponId
     * @return
     */
    private boolean repeatTakeCoupon(String userId,String couponId) {
        String takeId=tbCfTakeCouponDao.queryTakeByCouponId(userId,couponId);
        if(StringUtils.isBlank(takeId)) {
            return false;
        } else {
            return true;
        }
    }

    private void giveOutCoupon(String userId,TbCfCouponEntity tbCfCoupon) {

        //存储抢购记录
        TbCfTakeCouponEntity tbCfTakeCouponEntity = new TbCfTakeCouponEntity();
        tbCfTakeCouponEntity.setTakeId(IdUtil.createIdbyUUID());
        tbCfTakeCouponEntity.setCouponId(tbCfCoupon.getCouponId());
        tbCfTakeCouponEntity.setCreateTime(new Date());
        tbCfTakeCouponEntity.setUserId(userId);
        tbCfTakeCouponDao.save(tbCfTakeCouponEntity);
        //更改原优惠券记录
        update(tbCfCoupon);

    }
}
