package com.diaoyun.zion.chinafrica.service.impl;

import com.diaoyun.zion.chinafrica.dao.*;
import com.diaoyun.zion.chinafrica.entity.*;
import com.diaoyun.zion.chinafrica.enums.DeliveryStatusEnum;
import com.diaoyun.zion.chinafrica.enums.OrderStatusEnum;
import com.diaoyun.zion.chinafrica.service.SpiderService;
import com.diaoyun.zion.chinafrica.service.TbCfFeeService;
import com.diaoyun.zion.chinafrica.service.TbCfOrderService;
import com.diaoyun.zion.chinafrica.vo.TbCfCartItemDetailVo;
import com.diaoyun.zion.chinafrica.vo.TbCfOrderVo;
import com.diaoyun.zion.chinafrica.vo.TbCfUserInfoVo;
import com.diaoyun.zion.master.base.Result;
import com.diaoyun.zion.master.base.ResultCode;
import com.diaoyun.zion.master.base.StateConstant;
import com.diaoyun.zion.master.common.TokenManager;
import com.diaoyun.zion.master.util.CookieUtils;
import com.diaoyun.zion.master.util.IdUtil;
import com.diaoyun.zion.master.validator.Validator;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.net.URISyntaxException;
import java.util.*;

/**
 * Service实现类
 *
 * @author G
 * @date 2019-08-14 09:11:48
 */
@Service("tbCfOrderService")
public class TbCfOrderServiceImpl implements TbCfOrderService {
    @Autowired
    private TbCfOrderDao tbCfOrderDao;
    @Autowired
    private TbCfCouponDao tbCfCouponDao;
    @Autowired
    private TbCfAddressDao tbCfAddressDao;

    @Autowired
    private TbCfItemDetailDao tbCfItemDetailDao;

    @Autowired
    private TbCfTaxDao tbCfTaxDao;

    @Autowired
    private TbCfFeeService tbCfFeeService;
    @Autowired
    private SpiderService spiderService;

    @Resource(name = "redisTokenManager")
    private TokenManager tokenManager;

    @Autowired
    private HttpServletRequest request; //自动注入request

    @Override
    public TbCfOrderEntity queryObject(String orderId) {
        return tbCfOrderDao.queryObject(orderId);
    }

    @Override
    public List<TbCfOrderEntity> queryList(Map<String, Object> map) {
        return tbCfOrderDao.queryList(map);
    }

    @Override
    public int queryTotal(Map<String, Object> map) {
        return tbCfOrderDao.queryTotal(map);
    }

    @Override
    public int save(TbCfOrderEntity tbCfOrder) {
        tbCfOrder.setOrderId(IdUtil.createIdbyUUID());
        return tbCfOrderDao.save(tbCfOrder);
    }

    @Override
    public int update(TbCfOrderEntity tbCfOrder) {
        return tbCfOrderDao.update(tbCfOrder);
    }

    @Override
    public int delete(String orderId) {
        return tbCfOrderDao.delete(orderId);
    }

    @Override
    public int deleteBatch(String[] orderIds) {
        return tbCfOrderDao.deleteBatch(orderIds);
    }

    @Override
    public Result settleAccount() throws IOException, URISyntaxException {
        Result result = new Result();
        //获取用户
        String token = CookieUtils.getCookie(request, TokenManager.TOKEN);
        TbCfUserInfoVo tbCfUserInfoVo = tokenManager.validate(token);
        //获取购物车已勾选 商品
        List<TbCfCartItemDetailVo> tbCfCartItemDetailList = tbCfItemDetailDao.getCartItemList(tbCfUserInfoVo.getUserId(), StateConstant.VALID);

        if (tbCfCartItemDetailList != null && tbCfCartItemDetailList.size() > 0) {
            //获取订单数据
            TbCfOrderVo tbCfOrder = getOrderData(tbCfUserInfoVo, tbCfCartItemDetailList);
            result.setMessage("结算订单").setData(tbCfOrder);
        } else {
            result.setCode(ResultCode.ERROR).setMessage("购物车内没有选中商品");
        }
        return result;
    }

    @Override
    public Result placeOrder(TbCfOrderVo pageOrder) throws IOException, URISyntaxException {
        Result result = new Result();
        //获取用户
        String token = CookieUtils.getCookie(request, TokenManager.TOKEN);
        TbCfUserInfoVo tbCfUserInfoVo = tokenManager.validate(token);
        //获取购物车已勾选 商品
        List<TbCfCartItemDetailVo> tbCfCartItemDetailList = tbCfItemDetailDao.getCartItemList(tbCfUserInfoVo.getUserId(), StateConstant.VALID);

        if (tbCfCartItemDetailList != null && tbCfCartItemDetailList.size() > 0) {
            //获取下单的订单数据
            TbCfOrderVo definiteOrder = ensureOrder(tbCfUserInfoVo, pageOrder, tbCfCartItemDetailList);
            //持久化订单数据
            TbCfOrderEntity tbCfOrderEntity= new TbCfOrderEntity();
            BeanUtils.copyProperties(definiteOrder,tbCfOrderEntity);
            tbCfOrderDao.save(tbCfOrderEntity);
            result.setMessage("成功下单").setData(definiteOrder);
        } else {
            result.setCode(ResultCode.ERROR).setMessage("购物车内没有选中商品");
        }


        return result;
    }

    private TbCfOrderVo ensureOrder(TbCfUserInfoVo tbCfUserInfoVo, TbCfOrderVo pageOrder, List<TbCfCartItemDetailVo> tbCfCartItemDetailList) throws IOException, URISyntaxException {
        TbCfOrderVo definiteOrder = getGenericOrder(tbCfCartItemDetailList);
        //获取可以使用的优惠券，后续还有判断此订单是否可以使用
        List<TbCfCouponEntity> tbCfCouponList = tbCfCouponDao.queryUserAvailableCoupon(tbCfUserInfoVo.getUserId(), new Date());
        if (tbCfCouponList != null && tbCfCouponList.size() > 0) {
            //分开可使用和不可使用优惠券
            separateCoupon(definiteOrder, tbCfCouponList);
            if (definiteOrder.getUsableCouponList().size() > 0) {
                //设置默认的优惠券
                //验证订单内数据是否有效，目前需要验证优惠券，因为优惠券由页面传入，只有优惠券的选择才会影响到最后的金额。
                //也就是说，目前页面只需要传优惠券id就可以了。
                TbCfCouponEntity usableCoupon = checkCoupon(pageOrder.getCouponId(), definiteOrder.getUsableCouponList());
                if (usableCoupon != null) {
                    definiteOrder.setCouponId(usableCoupon.getCouponId());
                    definiteOrder.setCouponTitle(usableCoupon.getCouponTitle());
                    definiteOrder.setCouponPrice(usableCoupon.getDeductAmount());
                }
            }
        }

        //计算实际需要付款
        countRealityPay(definiteOrder);
        //获取订单地址
        String deliveryAddressId=pageOrder.getDeliveryAddressId();
        Validator.NOT_BLANK.validate(deliveryAddressId,"地址");
        TbCfAddressEntity tbCfAddressEntity=tbCfAddressDao.queryObject(deliveryAddressId);
        if(tbCfAddressEntity==null) {
            deliveryAddressId=null;
        }
        Validator.NOT_BLANK.validate(deliveryAddressId,"地址不存在");
        definiteOrder.setDeliveryAddressId(tbCfAddressEntity.getAddressId());
        definiteOrder.setDeliveryAddress(tbCfAddressEntity.getAddressDetail());
        definiteOrder.setDeliveryName(tbCfAddressEntity.getDeliveryName());
        definiteOrder.setDeliveryPhone(tbCfAddressEntity.getPhone());
        ///////////////////////////////地址 END//////////////////////////////
        definiteOrder.setDeliveryFlag(DeliveryStatusEnum.PROCESSING.getValue());

        definiteOrder.setOrderId(IdUtil.createIdbyUUID());
        definiteOrder.setOrderNo(IdUtil.createLongIdByDate());
        definiteOrder.setOrderTime(new Date());
        definiteOrder.setOrderStatus(OrderStatusEnum.PENDING_PAY.getValue());
        definiteOrder.setUserId(tbCfUserInfoVo.getUserId());
        definiteOrder.setUserName(tbCfUserInfoVo.getNick());
        definiteOrder.setPayStatus(OrderStatusEnum.PENDING_PAY.getValue());
        return definiteOrder;
    }

    /**
     * 获取通用的订单信息
     * @return
     */
    private TbCfOrderVo getGenericOrder(List<TbCfCartItemDetailVo> tbCfCartItemDetailList) throws IOException, URISyntaxException {
        TbCfOrderVo genericOrder = new TbCfOrderVo();
        genericOrder.setItemDetailList(tbCfCartItemDetailList);
        //初始商品总价
        BigDecimal itemsPrice = BigDecimal.valueOf(0.00);
        for (TbCfCartItemDetailVo tbCfCartItemDetailVo : tbCfCartItemDetailList) {
            BigDecimal itemNum = BigDecimal.valueOf(tbCfCartItemDetailVo.getItemNum());
            itemsPrice = itemsPrice.add(tbCfCartItemDetailVo.getItemPrice().multiply(itemNum));
        }
        /*获取人民币汇率 1美元换取人民币*/
        BigDecimal rate = spiderService.getExchangeRate(null);
        itemsPrice = itemsPrice.divide(rate, 2, BigDecimal.ROUND_UP);
        //itemsPrice=itemsPrice.setScale(2, BigDecimal.ROUND_UP);
        //System.out.println(itemsPrice);
        //计算手续费  TODO
        BigDecimal fee = countFee(itemsPrice);

        //税费
        BigDecimal tax = countTax(itemsPrice);
        //计算运费 TODO
        BigDecimal expressCost = new BigDecimal("0.01");
        //商品总价
        BigDecimal totalPrice = itemsPrice.add(fee).add(tax).add(expressCost);
        genericOrder.setTotalPrice(totalPrice);
        genericOrder.setItemsPrice(itemsPrice);
        genericOrder.setExpressCost(expressCost);
        genericOrder.setTax(tax);
        genericOrder.setFee(fee);
        return genericOrder;
    }

    /**
     * 验证订单优惠券
     *
     * @param couponId
     * @param usableCouponList
     * @return
     */
    private TbCfCouponEntity checkCoupon(String couponId, List<TbCfCouponEntity> usableCouponList) {
        TbCfCouponEntity result = null;
        for (TbCfCouponEntity tbCfCouponEntity : usableCouponList) {
            if (couponId.equals(tbCfCouponEntity.getCouponId())) {
                result = tbCfCouponEntity;
            }

        }
        return result;
    }


    private TbCfOrderVo getOrderData(TbCfUserInfoVo tbCfUserInfoVo, List<TbCfCartItemDetailVo> tbCfCartItemDetailList) throws IOException, URISyntaxException {
        TbCfOrderVo tbCfOrder = getGenericOrder(tbCfCartItemDetailList);
        //获取可以使用的优惠券，后续还有判断此订单是否可以使用
        List<TbCfCouponEntity> tbCfCouponList = tbCfCouponDao.queryUserAvailableCoupon(tbCfUserInfoVo.getUserId(), new Date());

        if (tbCfCouponList != null && tbCfCouponList.size() > 0) {
            //分开可使用和不可使用优惠券
            separateCoupon(tbCfOrder, tbCfCouponList);
            if (tbCfOrder.getUsableCouponList().size() > 0) {
                //设置默认的优惠券
                setDefaultCoupon(tbCfOrder);
            }
        }

        //计算实际需要付款
        countRealityPay(tbCfOrder);

        tbCfOrder.setUserId(tbCfUserInfoVo.getUserId());
        tbCfOrder.setUserName(tbCfUserInfoVo.getNick());
        return tbCfOrder;
    }

    private void countRealityPay(TbCfOrderVo tbCfOrder) {
        BigDecimal couponPrice=tbCfOrder.getCouponPrice();
        //实际需要支付款项
        if (couponPrice != null) {
            BigDecimal realityPay = tbCfOrder.getTotalPrice().subtract(couponPrice);
            //比0小则等于0
            if (realityPay.compareTo(BigDecimal.ZERO) < 0) {
                realityPay = BigDecimal.ZERO;
            }
            tbCfOrder.setRealityPay(realityPay);
        } else {
            tbCfOrder.setRealityPay(tbCfOrder.getTotalPrice());
        }
    }

    /**
     * 分开可使用和不可使用优惠券
     *
     * @param tbCfOrder
     * @param tbCfCouponList
     */
    private void separateCoupon(TbCfOrderVo tbCfOrder, List<TbCfCouponEntity> tbCfCouponList) {
        //可用优惠券
        List<TbCfCouponEntity> usableCouponList = new ArrayList<>();
        //不可用优惠券
        List<TbCfCouponEntity> unusableCouponList = new ArrayList<>();
        BigDecimal itemsPrice = tbCfOrder.getItemsPrice();
        for (TbCfCouponEntity tbCfCoupon : tbCfCouponList) {
            if (itemsPrice.compareTo(tbCfCoupon.getWithAmount()) >= 0) {
                usableCouponList.add(tbCfCoupon);
            } else {
                unusableCouponList.add(tbCfCoupon);
            }

        }
        tbCfOrder.setUsableCouponList(usableCouponList);
        tbCfOrder.setUnusableCouponList(unusableCouponList);
    }

    /**
     * 设置默认减免优惠券
     *
     * @param tbCfOrder
     */
    private void setDefaultCoupon(TbCfOrderVo tbCfOrder) {
        //可用优惠券
        List<TbCfCouponEntity> usableCouponList = tbCfOrder.getUsableCouponList();
        //根据抵扣金额排序 默认升序
        Collections.sort(usableCouponList, Comparator.comparing(TbCfCouponEntity::getDeductAmount));
        //倒序排列
        Collections.reverse(usableCouponList);
        tbCfOrder.setCouponId(usableCouponList.get(0).getCouponId());
        tbCfOrder.setCouponTitle(usableCouponList.get(0).getCouponTitle());
        tbCfOrder.setCouponPrice(usableCouponList.get(0).getDeductAmount());
    }

    /**
     * 计算税费
     *
     * @param itemsPrice
     * @return
     */
    private BigDecimal countTax(BigDecimal itemsPrice) {
        TbCfTaxEntity tbCfTax = tbCfTaxDao.getAvailableFee();
        BigDecimal tax = itemsPrice.multiply(tbCfTax.getTaxRate());
        tax = tax.setScale(2, BigDecimal.ROUND_UP);
        return tax;
    }

    /**
     * 计算手续费
     *
     * @param itemsPrice
     * @return
     */
    private BigDecimal countFee(BigDecimal itemsPrice) {
        TbCfFeeEntity tbCfFee = tbCfFeeService.getAvailableFee();
        BigDecimal fee = itemsPrice.multiply(tbCfFee.getFeePercent());
        fee = fee.setScale(2, BigDecimal.ROUND_UP);
        return fee;
    }
}
